/////////////////////////////////////////////////////////////////////////////////////////////
//BUTTON DEBOUNCE CALCULATOR / VERIFY
//Return true if the actual output value is true

//Return true if it's rising edge/falling edge
bool debounce_activate_edge(unsigned long* debounceStart)
{
  if(*debounceStart == ULONG_MAX){
    return false;
  }else if(*debounceStart == 0){
    *debounceStart = millis();
  }else if(millis()-*debounceStart>DEBOUNCE_DURATION){
    *debounceStart = ULONG_MAX;
    return true;
  }
  return false;
}


void debounce_deactivate(unsigned long* debounceStart){
  *debounceStart = 0;
}

/////////////////////////////////////////////////////////////////////////////////////////////
// Game RAM / characters

void graphic_flush(){
  lcd.clear();
  for(size_t x=0; x<16; x++){
    for(size_t y=0; y<2; y++){
      enum DisplayItem upperItem = (DisplayItem)((graphicRam[x/(8/2)][y*2]>>((x%(8/2))*2))&0x3);
      enum DisplayItem lowerItem = (DisplayItem)((graphicRam[x/(8/2)][y*2+1]>>((x%(8/2))*2))&0x3);
      if(upperItem>=GRAPHIC_ITEM_NUM)
        upperItem = GRAPHIC_ITEM_B;
      if(lowerItem>=GRAPHIC_ITEM_NUM)
        lowerItem = GRAPHIC_ITEM_B;
      lcd.setCursor(x, y);
      if(upperItem==0 && lowerItem==0)
        lcd.write(' ');
      else
        lcd.write(byte((uint8_t)upperItem*3+(uint8_t)lowerItem-1));
    }
  }
}

void graphic_generate_characters()
{
  for (size_t i=0; i<8; i++) {
    byte glyph[8];
    int upperIcon = (i+1)/3;
    int lowerIcon = (i+1)%3;

    memset(glyph, 0, sizeof(glyph));
    if(upperIcon==1)
      memcpy(&glyph[0], &block[0], 3);
    else if(upperIcon==2)
      memcpy(&glyph[0], &apple[0], 3);

    if(lowerIcon==1)
      memcpy(&glyph[4], &block[0], 3);
    else if(lowerIcon==2)
      memcpy(&glyph[4], &apple[0], 3);

    lcd.createChar(i, glyph);
  }
  delay(1); //Wait for the CGRAM to be written
}


void graphic_clear(){
  memset(graphicRam, 0, sizeof(graphicRam));
}

void graphic_add_item(uint8_t x, uint8_t y, enum DisplayItem item) {
  graphicRam[x/(8/2)][y] |= (uint8_t)item*(1<<((x%(8/2))*2));
}

void setLedColorHSV(int h, double s, double v) {
  //this is the algorithm to convert from RGB to HSV
  double r=0; 
  double g=0; 
  double b=0;

  double hf=h/60.0;

  int i=(int)floor(h/60.0);
  double f = h/60.0 - i;
  double pv = v * (1 - s);
  double qv = v * (1 - s*f);
  double tv = v * (1 - s * (1 - f));

  switch (i)
  {
  case 0: //rojo dominante
    r = v;
    g = tv;
    b = pv;
    break;
  case 1: //verde
    r = qv;
    g = v;
    b = pv;
    break;
  case 2: 
    r = pv;
    g = v;
    b = tv;
    break;
  case 3: //azul
    r = pv;
    g = qv;
    b = v;
    break;
  case 4:
    r = tv;
    g = pv;
    b = v;
    break;
  case 5: //rojo
    r = v;
    g = pv;
    b = qv;
    break;
  }

  //set each component to a integer value between 0 and 255
  int red=constrain((int)255*r,0,255);
  int green=constrain((int)255*g,0,255);
  int blue=constrain((int)255*b,0,255);

  setLedColor(red,green,blue);
}
void setLedColor(int redValue, int greenValue, int blueValue){
  analogWrite(redPin, redValue);
  analogWrite(greenPin, greenValue);
  analogWrite(bluePin, blueValue);
  }
